<?php

/**
 * @file
 * Contains \Drupal\niobi_app\Plugin\views\field\View.
 */

namespace Drupal\niobi_app\Plugin\views\field;

use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\niobi_app\Entity\NiobiApplication;
use Drupal\niobi_app\Entity\NiobiApplicationWorkflowStage;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Drupal\webform\Entity\WebformSubmission;

/**
 * @ViewsField("niobi_app_admin_status_actions")
 */
class NiobiApplicationAdminStatusActions extends FieldPluginBase {

  /**
   * @param $route
   * @param array $params
   * @return mixed
   */
  public static function checkRouteAccess($route, array $params) {
    $accessManager = \Drupal::service('access_manager');
    return $accessManager->checkNamedRoute($route, $params, \Drupal::currentUser());
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['view'] = ['default' => ''];
    $options['display'] = ['default' => 'default'];
    $options['arguments'] = ['default' => ''];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['niobi_app'] = [
      '#type' => 'details',
      '#title' => $this->t("View settings"),
      '#open' => TRUE,
    ];

  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    $this->addAdditionalFields();
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $output = '';
    /* @var $workflow \Drupal\niobi_app\Entity\NiobiApplicationWorkflow */
    /* @var $application \Drupal\niobi_app\Entity\NiobiApplication */
    $application = $values->_entity;
    $workflow = $application->getApplicationWorkflow();
    if ($workflow->isReadyToUse()) {
      $status = $application->get('field_application_status')->getValue();
      if (!empty($status)) {
        $status = $status[0]['value'];
        switch ($status) {
          case 'nomination':
            $output .= $this->addNominationLinks($application);
            break;
          case 'nomination_claim':
            $output .= $this->addNominationClaimLinks($application);
            break;
          case 'application':
            $output .= $this->addApplicationLinks($application);
            break;
          case 'review':
            $output .= $this->addReviewLinks($application);
            break;
          case 'decision':
            $output .= $this->addDecisionLinks($application);
            break;
          case 'complete':
            $output .= $this->addCompleteLinks($application);
            break;
        }
      }
    }
    else {
      \Drupal::messenger()->addError('The application has not been fully set up. Please contact the form administrator.');
      $output .= '<p><em>' . t('The application system is not fully set up.') . '</em></p>';
    }

    return ['#type' => 'markup', '#markup' => $output];
  }

  /**
   * @param NiobiApplication $application
   * @return string
   */
  public function addNominationLinks($application) {
    $ret = '';

    // Get the submitted form information
    $nomination_sub_ids = $application->get('field_nomination_submissions')->getValue();
    $complete_status = FALSE;
    if (!empty($nomination_sub_ids)) {
      $item = array_shift($nomination_sub_ids);
      $sub = WebformSubmission::load($item['target_id']);
      if ($sub) {
        $complete_status = $sub->isDraft() ? FALSE : TRUE;
      }
    }

    if ($complete_status) {
      $ret .= '<div class="bg-success">' . t('Nomination Complete') . '</div>';
    }
    else {
      $ret .= '<div class="bg-warning">' . t('Nomination Incomplete') . '</div>';
    }

    return $ret;
  }

  /**
   * @param NiobiApplication $application
   * @return string
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function addNominationClaimLinks($application) {
    return '<div class="bg-success">' . t('Nomination is complete and waiting to be claimed') . '</div>';
  }

  /**
   * @param NiobiApplication $application
   * @param NiobiApplicationWorkflowStage $stage
   * @param $stage_forms
   * @param $subs
   * @return bool
   */
  public function checkApplicationCompleteStatus($application, $stage, $stage_forms, $subs) {
    if (!empty($stage_forms['application']) && empty($subs['application'])) {
      return FALSE;
    }
    // Next, check the application submission is not a draft.
    elseif(!empty($stage_forms['application']) && !empty($subs['application'])) {
      $app_form = array_shift($stage_forms['application']);
      foreach ($subs['application'] as $sub_form => $sub_list) {
        if($app_form->getWebform()->id() === $sub_form) {
          // There should only be one application submission.
          $sub = array_shift($sub_list);
          if ($sub->isDraft()) {
            return FALSE;
          }
        }
      }
    }
    // We are in fact complete. Return TRUE.
    return TRUE;
  }

  /**
   * @param NiobiApplication $application
   * @param NiobiApplicationWorkflowStage $stage
   * @param $stage_forms
   * @param $subs
   * @return bool
   */
  public function checkRequiredAddendaCompleteStatus($application, $stage, $stage_forms, $subs) {
    if(!empty($stage_forms['required_addenda']) && empty($subs['application'])) {
      return FALSE;
    }
    elseif(!empty($stage_forms['required_addenda']) && !empty($subs['application'])) {
      foreach($stage_forms['required_addenda'] as $addenda_form) {
        $addenda_form_id = $addenda_form->getWebform()->id();
        // A required form is missing a submission.
        if (!isset($subs['application'][$addenda_form_id])) {
          return FALSE;
        }
        else {
          $sub = array_shift($subs['application'][$addenda_form_id]);
          if ($sub->isDraft()) {
            return FALSE;
          }
        }
      }
    }
    // We are in fact complete. Return TRUE.
    return TRUE;
  }

  /**
   * @param NiobiApplication $application
   * @param NiobiApplicationWorkflowStage $stage
   * @param $stage_forms
   * @param $subs
   * @return int
   */
  public function getSupportLetterCompleteCount($application, $stage, $stage_forms, $subs) {
    if(!empty($stage_forms['support']) && !empty($subs['application'])) {
      foreach($stage_forms['support'] as $support_form) {
        if ($support_form) {
          $support_form_id = $support_form->getWebform()->id();
          // A required form is missing a submission.
          if (isset($subs['application'][$support_form_id])) {
            $count = 0;
            $info = $stage->getSupportLetterInformation();
            if ($info['form']) {
              $count = 0;
              if (isset($subs['application'][$support_form_id])) {
                foreach ($subs['application'][$support_form_id] as $item) {
                  if (!$item->isDraft()) {
                    $count += 1;
                  }
                }
              }
            }
            return $count;
          }
        }
      }
    }
    // We have a condition that prevents counting, so return zero.
    return 0;
  }

  /**
   * @param $application
   * @param $stage
   * @param $stage_forms
   * @param $subs
   * @return bool
   */
  public function checkSupportLetterCompleteStatus($application, $stage, $stage_forms, $subs) {
    if(!empty($stage_forms['support']) && empty($subs['application'])) {
      return FALSE;
    }
    elseif(!empty($stage_forms['support']) && !empty($subs['application'])) {
      foreach($stage_forms['support'] as $support_form) {
        if ($support_form) {
          $support_form_id = $support_form->getWebform()->id();
          // A required form is missing a submission.
          if (!isset($subs['application'][$support_form_id])) {
            return FALSE;
          }
          else {
            $count = 0;
            $info = $stage->getSupportLetterInformation();
            if ($info['form']) {
              $count_req = !empty($info['count']) ? $info['count'] : 1;
              $count = $this->getSupportLetterCompleteCount($application, $stage, $stage_forms, $subs);
            }
            if ($count < $count_req) {
              return FALSE;
            }
          }
        }
      }
    }
    // We are in fact complete. Return TRUE.
    return TRUE;
  }

  /**
   * @param NiobiApplication $application
   */
  public function applicationCompleteStatus($application) {
    $stage = $application->getCurrentApplicationStage();
    $stage_forms = $stage->getAllForms();
    $subs = $application->getAllSubmissionsbyWebform();

    // First, the application form must exist, and be completed.
    if (!$this->checkApplicationCompleteStatus($application, $stage, $stage_forms, $subs)) {
      return FALSE;
    }

    // Now we check required addenda
    if (!$this->checkRequiredAddendaCompleteStatus($application, $stage, $stage_forms, $subs)) {
      return FALSE;
    }

    // Finally, check the support letters.
    if (!$this->checkSupportLetterCompleteStatus($application, $stage, $stage_forms, $subs)) {
      return FALSE;
    }

    // We are in fact complete. Return TRUE.
    return TRUE;
  }

  /**
   * @param NiobiApplication $application
   * @return string
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function addApplicationLinks($application) {
    $ret = '';

    $complete_status = $this->applicationCompleteStatus($application);

    $workflow = $application->getApplicationWorkflow();
    $stage = $application->getCurrentApplicationStage();
    $stage_forms = $stage->getAllForms();
    $subs = $application->getAllSubmissionsbyWebform();

    if ($complete_status) {
      $ret .= '<div class="bg-success">' . t('Application is complete and ready to submit.') . '</div>';
      $params = [
        'niobi_application_workflow' => $workflow->id(),
        'niobi_application' => $application->id(),
      ];
      $options = [
        'attributes' => [
          'class' => 'btn btn-sm btn-success'
        ],
      ];
      $route = 'niobi_app.move_to_review';
      $access = $this->checkRouteAccess($route, $params);
      if ($access) {
        $url = Url::fromRoute($route, $params, $options);
        $link = Link::fromTextAndUrl('Submit for Review as Admin >', $url);
        $ret .= '<p>' . $link->toString() . '</p>';
      }
    }
    else {
      $ret .= '<div class="bg-warning">' . t('The following items are not complete') . ':';
      $ret .= '<ul>';
      if (!$this->checkApplicationCompleteStatus($application, $stage, $stage_forms, $subs)) {
        $ret .= '<li>' . t('Application') . '</li>';
      }
      if (!$this->checkRequiredAddendaCompleteStatus($application, $stage, $stage_forms, $subs)) {
        $ret .= '<li>' . t('Required Addenda') . '</li>';
      }
      if (!$this->checkSupportLetterCompleteStatus($application, $stage, $stage_forms, $subs)) {
        $info = $stage->getSupportLetterInformation();
        if ($info['form'] && $info['count'] > 0) {
          $name = !empty($info['name']) ? $info['name'] : t('Support Letters');
          $count_req = !empty($info['count']) ? $info['count'] : 1;
          $count = $this->getSupportLetterCompleteCount($application, $stage, $stage_forms, $subs);
          if (isset($subs['application'][$info['form']->id()])) {
            foreach($subs['application'][$info['form']->id()] as $item) {
              if (!$item->isDraft()) {
                $count += 1;
              }
            }
          }
          $ret .= '<li>' . t('%s: %count of %total required',
              [
                '%s' => $name,
                '%count' => $count,
                '%total' => $count_req,
              ]
            ) . '</li>';
        }
      }
      $ret .= '</ul></div>';
    }

    return $ret;
  }

  /**
   * @param NiobiApplication $application
   * @return string
   */
  public function addReviewLinks($application) {
    $ret = '<div class="bg-success">' . t('The application is under review.') . '</div>';
    $options = [
      'attributes' => [
        'class' => 'btn btn-sm btn-success'
      ],
    ];
    $params = ['niobi_application' => $application->id()];
    $route = 'niobi_app.move_to_decision';
    $access = $this->checkRouteAccess($route, $params);
    if ($access) {
      $url = Url::fromRoute($route, $params, $options);
      $link = Link::fromTextAndUrl('Move to Decision Stage >', $url);
      $ret .= $link->toString()->getGeneratedLink();
    }
    return $ret;
  }

  /**
   * @param NiobiApplication $application
   * @return string
   */
  public function addDecisionLinks($application) {
    $ret = '<div class="bg-success">' . t('The application is ready for an administrative decision.') . '</div>';

    $decisions = $application->getCurrentApplicationStage()->getDecisionsAsNames();

    foreach ($decisions as $key => $decision) {
      $options = [
        'attributes' => [
          'class' => 'btn btn-sm btn-info'
        ],
      ];
      $params = ['niobi_application' => $application->id(), 'decision_plugin' => $key];
      $route = 'niobi_app.decision_form';
      $access = $this->checkRouteAccess($route, $params);
      if ($access) {
        $url = Url::fromRoute($route, $params, $options);
        $link = Link::fromTextAndUrl($decision, $url);
        $ret .= $link->toString()->getGeneratedLink() . '<br>';
      }
    }
    return $ret;
  }

  /**
   * @param NiobiApplication $application
   * @return string
   */
  public function addCompleteLinks($application) {
    $ret = '<p>' . t('Application is complete.') . '</p>';
    $decision = $application->get('field_decision')->getValue();
    $decision = !empty($decision) ? $decision[0]['value'] : '';
    $ret .= '<p>' . t('Decision: %decision', ['%decision' => $decision]) . '</p>';
    return $ret;
  }

}
